@@ -10,11 +10,9 @@ class Archive:public File
   private:
     bool IsSignature(byte *D);
     void UpdateLatestTime(FileHeader *CurBlock);
-    void Protect(int RecSectors);
     void ConvertNameCase(char *Name);
     void ConvertNameCase(wchar *Name);
     void ConvertUnknownHeader();
-    bool AddArcComment(char *NameToShow);
     int ReadOldHeader();
     void PrepareExtraTime(FileHeader *hd,EXTTIME_MODE etm,EXTTIME_MODE etc,EXTTIME_MODE eta,EXTTIME_MODE etarc,Array<byte> &TimeData);
 
@@ -62,7 +60,7 @@ class Archive:public File
     void WriteEndBlock(bool CloseVolume);
     void CopyFileRecord(Archive &Src);
     void CopyArchiveData(Archive &Src);
-    bool GetComment(Array<byte> &CmtData);
+    bool GetComment(Array<byte> *CmtData,Array<wchar> *CmtDataW);
     void ViewComment();
     void ViewFileComment();
     void SetLatestTime(RarTime *NewTime);
@@ -71,14 +69,12 @@ class Archive:public File
     bool IsArcDir();
     bool IsArcLabel();
     void ConvertAttributes();
-    int LhdSize();
-    int LhdExtraSize();
     int GetRecoverySize(bool Required);
     void VolSubtractHeaderSize(int SubSize);
     void AddSubData(byte *SrcData,int DataSize,File *SrcFile,char *Name,bool AllowSplit);
     bool ReadSubData(Array<byte> *UnpData,File *DestFile);
     int GetHeaderType() {return(CurHeaderType);};
-    int ReadCommentData(Array<byte> &CmtData);
+    int ReadCommentData(Array<byte> *CmtData,Array<wchar> *CmtDataW);
     void WriteCommentData(byte *Data,int DataSize,bool FileComment);
     RAROptions* GetRAROptions() {return(Cmd);}
     void SetSilentOpen(bool Mode) {SilentOpen=Mode;}
