@@ -44,7 +44,7 @@ void CommandData::Close()
 }
 
 
-#if !defined(SFX_MODULE) && !defined(_WIN_CE)
+#if !defined(SFX_MODULE)
 void CommandData::ParseArg(char *Arg,wchar *ArgW)
 {
   if (IsSwitch(*Arg) && !NoMoreSwitches)
@@ -55,10 +55,10 @@ void CommandData::ParseArg(char *Arg,wchar *ArgW)
   else
     if (*Command==0)
     {
-      strncpy(Command,Arg,sizeof(Command));
+      strncpyz(Command,Arg,ASIZE(Command));
       if (ArgW!=NULL)
         strncpyw(CommandW,ArgW,sizeof(CommandW)/sizeof(CommandW[0]));
-      if (toupper(*Command)=='S')
+      if (etoupper(*Command)=='S')
       {
         const char *SFXName=Command[1] ? Command+1:DefSFXName;
         if (PointToName(SFXName)!=SFXName || FileExist(SFXName))
@@ -67,7 +67,7 @@ void CommandData::ParseArg(char *Arg,wchar *ArgW)
           GetConfigName(SFXName,SFXModule,true);
       }
 #ifndef GUI
-      *Command=toupper(*Command);
+      *Command=etoupper(*Command);
       if (*Command!='I' && *Command!='S')
         strupper(Command);
 #endif
@@ -75,19 +75,23 @@ void CommandData::ParseArg(char *Arg,wchar *ArgW)
     else
       if (*ArcName==0)
       {
-        strncpy(ArcName,Arg,sizeof(ArcName));
+        strncpyz(ArcName,Arg,ASIZE(ArcName));
         if (ArgW!=NULL)
-          strncpyw(ArcNameW,ArgW,sizeof(ArcNameW)/sizeof(ArcNameW[0]));
+          strncpyzw(ArcNameW,ArgW,ASIZE(ArcNameW));
       }
       else
       {
         int Length=strlen(Arg);
         char EndChar=Arg[Length-1];
-        char CmdChar=toupper(*Command);
+        char CmdChar=etoupper(*Command);
         bool Add=strchr("AFUM",CmdChar)!=NULL;
         bool Extract=CmdChar=='X' || CmdChar=='E';
         if ((IsDriveDiv(EndChar) || IsPathDiv(EndChar)) && !Add)
-          strcpy(ExtrPath,Arg);
+        {
+          strncpyz(ExtrPath,Arg,ASIZE(ExtrPath));
+          if (ArgW!=NULL)
+            strncpyzw(ExtrPathW,ArgW,ASIZE(ExtrPathW));
+        }
         else
           if ((Add || CmdChar=='T') && *Arg!='@')
             FileArgs->AddString(Arg);
@@ -97,8 +101,19 @@ void CommandData::ParseArg(char *Arg,wchar *ArgW)
             bool Found=FindFile::FastFind(Arg,NULL,&FileData);
             if (!Found && *Arg=='@' && !IsWildcard(Arg))
             {
-              ReadTextFile(Arg+1,FileArgs,false,true,true,true,true);
               FileLists=true;
+
+              RAR_CHARSET Charset=FilelistCharset;
+
+#if defined(_WIN_32) && !defined(GUI)
+              // for compatibility reasons we use OEM encoding
+              // in Win32 console version by default
+
+              if (Charset==RCH_DEFAULT)
+                Charset=RCH_OEM;
+#endif
+
+              ReadTextFile(Arg+1,FileArgs,false,true,Charset,true,true);
             }
             else
               if (Found && FileData.IsDir && Extract && *ExtrPath==0)
@@ -118,7 +133,7 @@ void CommandData::ParseDone()
 {
   if (FileArgs->ItemsCount()==0 && !FileLists)
     FileArgs->AddString(MASKALL);
-  char CmdChar=toupper(*Command);
+  char CmdChar=etoupper(*Command);
   bool Extract=CmdChar=='X' || CmdChar=='E';
   if (Test && Extract)
     Test=false;
@@ -136,7 +151,11 @@ void CommandData::ParseEnvVar()
 #endif
 
 
-#if !defined(GUI) && !defined(SFX_MODULE)
+
+// return 'false' if -cfg- is present and preprocess switches
+// which must be processed before the rest of command line
+
+#ifndef SFX_MODULE
 bool CommandData::IsConfigEnabled(int argc,char *argv[])
 {
   bool ConfigEnabled=true;
@@ -145,11 +164,20 @@ bool CommandData::IsConfigEnabled(int argc,char *argv[])
     {
       if (stricomp(&argv[I][1],"cfg-")==0)
         ConfigEnabled=false;
+#ifndef GUI
       if (strnicomp(&argv[I][1],"ilog",4)==0)
       {
+        // ensure that correct log file name is already set
+        // if we need to report an error when processing the command line
         ProcessSwitch(&argv[I][1]);
         InitLogOptions(LogName);
       }
+#endif
+      if (strnicomp(&argv[I][1],"sc",2)==0)
+      {
+        // process -cs before reading any file lists
+        ProcessSwitch(&argv[I][1]);
+      }
     }
   return(ConfigEnabled);
 }
@@ -164,8 +192,12 @@ void CommandData::ReadConfig(int argc,char *argv[])
   {
     char *Str;
     while ((Str=List.GetString())!=NULL)
+    {
+      while (isspace(*Str))
+        Str++;
       if (strnicomp(Str,"switches=",9)==0)
         ProcessSwitchesString(Str+9);
+    }
   }
 }
 #endif
@@ -193,16 +225,16 @@ void CommandData::ProcessSwitchesString(char *Str)
 #endif
 
 
-#if !defined(SFX_MODULE) && !defined(_WIN_CE)
+#if !defined(SFX_MODULE)
 void CommandData::ProcessSwitch(char *Switch)
 {
 
-  switch(toupper(Switch[0]))
+  switch(etoupper(Switch[0]))
   {
     case 'I':
       if (strnicomp(&Switch[1],"LOG",3)==0)
       {
-        strncpy(LogName,Switch[4] ? Switch+4:DefLogName,sizeof(LogName));
+        strncpyz(LogName,Switch[4] ? Switch+4:DefLogName,ASIZE(LogName));
         break;
       }
       if (stricomp(&Switch[1],"SND")==0)
@@ -217,7 +249,7 @@ void CommandData::ProcessSwitch(char *Switch)
       }
       if (strnicomp(&Switch[1],"EML",3)==0)
       {
-        strncpy(EmailTo,Switch[4] ? Switch+4:"@",sizeof(EmailTo));
+        strncpyz(EmailTo,Switch[4] ? Switch+4:"@",ASIZE(EmailTo));
         EmailTo[sizeof(EmailTo)-1]=0;
         break;
       }
@@ -226,10 +258,10 @@ void CommandData::ProcessSwitch(char *Switch)
         MsgStream=MSG_NULL;
         break;
       }
-      if (toupper(Switch[1])=='D')
+      if (etoupper(Switch[1])=='D')
       {
         for (int I=2;Switch[I]!=0;I++)
-          switch(toupper(Switch[I]))
+          switch(etoupper(Switch[I]))
           {
             case 'Q':
               MsgStream=MSG_ERRONLY;
@@ -253,7 +285,7 @@ void CommandData::ProcessSwitch(char *Switch)
       }
       break;
     case 'T':
-      switch(toupper(Switch[1]))
+      switch(etoupper(Switch[1]))
       {
         case 'K':
           ArcTime=ARCTIME_KEEP;
@@ -289,7 +321,7 @@ void CommandData::ProcessSwitch(char *Switch)
                 Mode=(EXTTIME_MODE)(Switch[3]-'0');
               if (Switch[3]=='-')
                 Mode=EXTTIME_NONE;
-              switch(toupper(Switch[2]))
+              switch(etoupper(Switch[2]))
               {
                 case 'M':
                   xmtime=Mode;
@@ -319,7 +351,7 @@ void CommandData::ProcessSwitch(char *Switch)
       }
       break;
     case 'A':
-      switch(toupper(Switch[1]))
+      switch(etoupper(Switch[1]))
       {
         case 'C':
           ClearArc=true;
@@ -333,7 +365,7 @@ void CommandData::ProcessSwitch(char *Switch)
           else
           {
             GenerateArcName=true;
-            strncpy(GenerateMask,Switch+2,sizeof(GenerateMask));
+            strncpyz(GenerateMask,Switch+2,ASIZE(GenerateMask));
           }
           break;
         case 'N': //reserved for archive name
@@ -347,11 +379,14 @@ void CommandData::ProcessSwitch(char *Switch)
         case 'S':
           SyncFiles=true;
           break;
+        default:
+          BadSwitch(Switch);
+          break;
       }
       break;
     case 'D':
       if (Switch[2]==0)
-        switch(toupper(Switch[1]))
+        switch(etoupper(Switch[1]))
         {
           case 'S':
             DisableSortSolid=true;
@@ -365,7 +400,7 @@ void CommandData::ProcessSwitch(char *Switch)
         }
       break;
     case 'O':
-      switch(toupper(Switch[1]))
+      switch(etoupper(Switch[1]))
       {
         case '+':
           Overwrite=OVERWRITE_ALL;
@@ -373,6 +408,9 @@ void CommandData::ProcessSwitch(char *Switch)
         case '-':
           Overwrite=OVERWRITE_NONE;
           break;
+        case 'R':
+          Overwrite=OVERWRITE_AUTORENAME;
+          break;
         case 'W':
           ProcessOwners=true;
           break;
@@ -395,7 +433,7 @@ void CommandData::ProcessSwitch(char *Switch)
       }
       break;
     case 'R':
-      switch(toupper(Switch[1]))
+      switch(etoupper(Switch[1]))
       {
         case 0:
           Recurse=RECURSE_ALWAYS;
@@ -406,6 +444,7 @@ void CommandData::ProcessSwitch(char *Switch)
         case '0':
           Recurse=RECURSE_WILDCARDS;
           break;
+#ifndef _WIN_CE
         case 'I':
           {
             Priority=atoi(Switch+2);
@@ -418,6 +457,7 @@ void CommandData::ProcessSwitch(char *Switch)
             SetPriority(Priority);
           }
           break;
+#endif
       }
       break;
     case 'Y':
@@ -427,15 +467,27 @@ void CommandData::ProcessSwitch(char *Switch)
     case 'X':
       if (Switch[1]!=0)
       {
-        StringList *Args=toupper(Switch[0])=='N' ? InclArgs:ExclArgs;
+        StringList *Args=etoupper(Switch[0])=='N' ? InclArgs:ExclArgs;
         if (Switch[1]=='@' && !IsWildcard(Switch))
-          ReadTextFile(Switch+2,Args,false,true,true,true,true);
+        {
+          RAR_CHARSET Charset=FilelistCharset;
+
+#if defined(_WIN_32) && !defined(GUI)
+          // for compatibility reasons we use OEM encoding
+          // in Win32 console version by default
+
+          if (Charset==RCH_DEFAULT)
+            Charset=RCH_OEM;
+#endif
+
+          ReadTextFile(Switch+2,Args,false,true,Charset,true,true);
+        }
         else
           Args->AddString(Switch+1);
       }
       break;
     case 'E':
-      switch(toupper(Switch[1]))
+      switch(etoupper(Switch[1]))
       {
         case 'P':
           switch(Switch[2])
@@ -481,14 +533,14 @@ void CommandData::ProcessSwitch(char *Switch)
         eprintf("\n");
       }
       else
-        strncpy(Password,Switch+1,sizeof(Password));
+        strncpyz(Password,Switch+1,ASIZE(Password));
       break;
     case 'H':
-      if (toupper(Switch[1])=='P')
+      if (etoupper(Switch[1])=='P')
       {
         EncryptHeaders=true;
         if (Switch[2]!=0)
-          strncpy(Password,Switch+2,sizeof(Password));
+          strncpyz(Password,Switch+2,ASIZE(Password));
         else
           if (*Password==0)
           {
@@ -498,10 +550,10 @@ void CommandData::ProcessSwitch(char *Switch)
       }
       break;
     case 'Z':
-      strncpy(CommentFile,Switch[1]!=0 ? Switch+1:"stdin",sizeof(CommentFile));
+      strncpyz(CommentFile,Switch[1]!=0 ? Switch+1:"stdin",ASIZE(CommentFile));
       break;
     case 'M':
-      switch(toupper(Switch[1]))
+      switch(etoupper(Switch[1]))
       {
         case 'C':
           {
@@ -527,7 +579,7 @@ void CommandData::ProcessSwitch(char *Switch)
                   while (isdigit(*Str))
                     Str++;
                 }
-                switch(toupper(*(Str++)))
+                switch(etoupper(*(Str++)))
                 {
                   case 'T': Type=FILTER_PPM;         break;
                   case 'E': Type=FILTER_E8;          break;
@@ -550,7 +602,7 @@ void CommandData::ProcessSwitch(char *Switch)
         case 'D':
           {
             if ((WinSize=atoi(&Switch[2]))==0)
-              WinSize=0x10000<<(toupper(Switch[2])-'A');
+              WinSize=0x10000<<(etoupper(Switch[2])-'A');
             else
               WinSize*=1024;
             if (!CheckWinSize())
@@ -584,6 +636,16 @@ void CommandData::ProcessSwitch(char *Switch)
             }
           }
           break;
+#ifdef PACK_SMP
+        case 'T':
+          Threads=atoi(Switch+2);
+          if (Threads>MaxSearchThreads)
+            BadSwitch(Switch);
+          else
+          {
+          }
+          break;
+#endif
         default:
           Method=Switch[1]-'0';
           if (Method>5 || Method<0)
@@ -592,7 +654,7 @@ void CommandData::ProcessSwitch(char *Switch)
       }
       break;
     case 'V':
-      switch(toupper(Switch[1]))
+      switch(etoupper(Switch[1]))
       {
 #ifdef _WIN_32
         case 'D':
@@ -606,7 +668,7 @@ void CommandData::ProcessSwitch(char *Switch)
           VolumePause=true;
           break;
         case 'E':
-          if (toupper(Switch[2])=='R')
+          if (etoupper(Switch[2])=='R')
             VersionControl=atoi(Switch+3)+1;
           break;
         case '-':
@@ -677,13 +739,17 @@ void CommandData::ProcessSwitch(char *Switch)
     case 'F':
       if (Switch[1]==0)
         FreshFiles=true;
+      else
+        BadSwitch(Switch);
       break;
     case 'U':
       if (Switch[1]==0)
         UpdateFiles=true;
+      else
+        BadSwitch(Switch);
       break;
     case 'W':
-      strncpy(TempPath,&Switch[1],sizeof(TempPath)-1);
+      strncpyz(TempPath,&Switch[1],ASIZE(TempPath));
       AddEndSlash(TempPath);
       break;
     case 'S':
@@ -701,7 +767,7 @@ void CommandData::ProcessSwitch(char *Switch)
         SolidCount=atoi(&Switch[1]);
       }
       else
-        switch(toupper(Switch[1]))
+        switch(etoupper(Switch[1]))
         {
           case 0:
             Solid|=SOLID_NORMAL;
@@ -718,11 +784,56 @@ void CommandData::ProcessSwitch(char *Switch)
           case 'D':
             Solid|=SOLID_VOLUME_DEPENDENT;
             break;
+          case 'L':
+            if (isdigit(Switch[2]))
+              FileSizeLess=atoil(Switch+2);
+            break;
+          case 'M':
+            if (isdigit(Switch[2]))
+              FileSizeMore=atoil(Switch+2);
+            break;
+          case 'C':
+            {
+              RAR_CHARSET rch;
+              switch(etoupper(Switch[2]))
+              {
+                case 'A':
+                  rch=RCH_ANSI;
+                  break;
+                case 'O':
+                  rch=RCH_OEM;
+                  break;
+                case 'U':
+                  rch=RCH_UNICODE;
+                  break;
+                default :
+                  BadSwitch(Switch);
+                  break;
+              };
+              if (Switch[3]==0)
+                CommentCharset=FilelistCharset=rch;
+              else
+                for (int I=3;Switch[I]!=0;I++)
+                  switch(etoupper(Switch[I]))
+                  {
+                    case 'C':
+                      CommentCharset=rch;
+                      break;
+                    case 'L':
+                      FilelistCharset=rch;
+                      break;
+                    default:
+                      BadSwitch(Switch);
+                      break;
+                  }
+            }
+            break;
+
         }
       break;
     case 'C':
       if (Switch[2]==0)
-        switch(toupper(Switch[1]))
+        switch(etoupper(Switch[1]))
         {
           case '-':
             DisableComment=true;
@@ -736,7 +847,7 @@ void CommandData::ProcessSwitch(char *Switch)
         }
       break;
     case 'K':
-      switch(toupper(Switch[1]))
+      switch(etoupper(Switch[1]))
       {
         case 'B':
           KeepBroken=true;
@@ -759,7 +870,7 @@ void CommandData::ProcessSwitch(char *Switch)
 #endif
 
 
-#if !defined(SFX_MODULE) && !defined(_WIN_CE)
+#ifndef SFX_MODULE
 void CommandData::BadSwitch(char *Switch)
 {
   mprintf(St(MUnknownOption),Switch);
@@ -810,26 +921,27 @@ void CommandData::OutHelp()
     MCHelpSwAP,MCHelpSwAVm,MCHelpSwCm,MCHelpSwCFGm,MCHelpSwCL,MCHelpSwCU,
     MCHelpSwDH,MCHelpSwEP,MCHelpSwEP3,MCHelpSwF,MCHelpSwIDP,MCHelpSwIERR,
     MCHelpSwINUL,MCHelpSwIOFF,MCHelpSwKB,MCHelpSwN,MCHelpSwNa,MCHelpSwNal,
-    MCHelpSwOp,MCHelpSwOm,MCHelpSwOC,MCHelpSwOW,MCHelpSwP,MCHelpSwPm,
-    MCHelpSwR,MCHelpSwRI,MCHelpSwTA,MCHelpSwTB,MCHelpSwTN,MCHelpSwTO,
-    MCHelpSwTS,MCHelpSwU,MCHelpSwVUnr,MCHelpSwVER,MCHelpSwVP,MCHelpSwX,
-    MCHelpSwXa,MCHelpSwXal,MCHelpSwY
+    MCHelpSwOp,MCHelpSwOm,MCHelpSwOC,MCHelpSwOR,MCHelpSwOW,MCHelpSwP,
+    MCHelpSwPm,MCHelpSwR,MCHelpSwRI,MCHelpSwSL,MCHelpSwSM,MCHelpSwTA,
+    MCHelpSwTB,MCHelpSwTN,MCHelpSwTO,MCHelpSwTS,MCHelpSwU,MCHelpSwVUnr,
+    MCHelpSwVER,MCHelpSwVP,MCHelpSwX,MCHelpSwXa,MCHelpSwXal,MCHelpSwY
 #else
     MRARTitle1,MRARTitle2,MCHelpCmd,MCHelpCmdA,MCHelpCmdC,MCHelpCmdCF,
-    MCHelpCmdCW,MCHelpCmdD,MCHelpCmdE,MCHelpCmdF,MCHelpCmdI,MCHelpCmdK,
-    MCHelpCmdL,MCHelpCmdM,MCHelpCmdP,MCHelpCmdR,MCHelpCmdRC,MCHelpCmdRN,
-    MCHelpCmdRR,MCHelpCmdRV,MCHelpCmdS,MCHelpCmdT,MCHelpCmdU,MCHelpCmdV,
-    MCHelpCmdX,MCHelpSw,MCHelpSwm,MCHelpSwAC,MCHelpSwAD,MCHelpSwAG,
+    MCHelpCmdCH,MCHelpCmdCW,MCHelpCmdD,MCHelpCmdE,MCHelpCmdF,MCHelpCmdI,
+    MCHelpCmdK,MCHelpCmdL,MCHelpCmdM,MCHelpCmdP,MCHelpCmdR,MCHelpCmdRC,
+    MCHelpCmdRN,MCHelpCmdRR,MCHelpCmdRV,MCHelpCmdS,MCHelpCmdT,MCHelpCmdU,
+    MCHelpCmdV,MCHelpCmdX,MCHelpSw,MCHelpSwm,MCHelpSwAC,MCHelpSwAD,MCHelpSwAG,
     MCHelpSwAO,MCHelpSwAP,MCHelpSwAS,MCHelpSwAV,MCHelpSwAVm,MCHelpSwCm,
     MCHelpSwCFGm,MCHelpSwCL,MCHelpSwCU,MCHelpSwDF,MCHelpSwDH,MCHelpSwDS,
     MCHelpSwEa,MCHelpSwED,MCHelpSwEE,MCHelpSwEN,MCHelpSwEP,MCHelpSwEP1,
-    MCHelpSwEP2,MCHelpSwEP3,MCHelpSwF,MCHelpSwHP,MCHelpSwIDP,MCHelpSwIEML,
-    MCHelpSwIERR,MCHelpSwILOG,MCHelpSwINUL,MCHelpSwIOFF,MCHelpSwISND,
-    MCHelpSwK,MCHelpSwKB,MCHelpSwMn,MCHelpSwMC,MCHelpSwMD,MCHelpSwMS,
-    MCHelpSwN,MCHelpSwNa,MCHelpSwNal,MCHelpSwOp,MCHelpSwOm,MCHelpSwOC,
-    MCHelpSwOL,MCHelpSwOS,MCHelpSwOW,MCHelpSwP,MCHelpSwPm,MCHelpSwR,
-    MCHelpSwR0,MCHelpSwRI,MCHelpSwRR,MCHelpSwRV,MCHelpSwS,MCHelpSwSm,
-    MCHelpSwSFX,MCHelpSwSI,MCHelpSwT,MCHelpSwTA,MCHelpSwTB,MCHelpSwTK,
+    MCHelpSwEP2,MCHelpSwEP3,MCHelpSwF,MCHelpSwHP,MCHelpSwIDP,
+    MCHelpSwIEML,MCHelpSwIERR,MCHelpSwILOG,MCHelpSwINUL,MCHelpSwIOFF,
+    MCHelpSwISND,MCHelpSwK,MCHelpSwKB,MCHelpSwMn,MCHelpSwMC,MCHelpSwMD,
+    MCHelpSwMS,MCHelpSwMT,MCHelpSwN,MCHelpSwNa,MCHelpSwNal,MCHelpSwOp,
+    MCHelpSwOm,MCHelpSwOC,MCHelpSwOL,MCHelpSwOR,MCHelpSwOS,MCHelpSwOW,
+    MCHelpSwP,MCHelpSwPm,MCHelpSwR,MCHelpSwR0,MCHelpSwRI,MCHelpSwRR,
+    MCHelpSwRV,MCHelpSwS,MCHelpSwSm,MCHelpSwSC,MCHelpSwSFX,MCHelpSwSI,
+    MCHelpSwSL,MCHelpSwSM,MCHelpSwT,MCHelpSwTA,MCHelpSwTB,MCHelpSwTK,
     MCHelpSwTL,MCHelpSwTN,MCHelpSwTO,MCHelpSwTS,MCHelpSwU,MCHelpSwV,
     MCHelpSwVn,MCHelpSwVD,MCHelpSwVER,MCHelpSwVN,MCHelpSwVP,MCHelpSwW,
     MCHelpSwX,MCHelpSwXa,MCHelpSwXal,MCHelpSwY,MCHelpSwZ
@@ -845,7 +957,7 @@ void CommandData::OutHelp()
 #endif
 #ifndef _WIN_32
     static MSGID Win32Only[]={
-      MCHelpSwIEML,MCHelpSwVD,MCHelpSwAC,MCHelpSwAO,MCHelpSwOS,MCHelpSwIOFF,
+      MCHelpSwIEML,MCHelpSwVD,MCHelpSwAO,MCHelpSwOS,MCHelpSwIOFF,
       MCHelpSwEP2,MCHelpSwOC
     };
     bool Found=false;
@@ -862,15 +974,23 @@ void CommandData::OutHelp()
     if (Help[I]==MCHelpSwOW)
       continue;
 #endif
+#if !defined(_WIN_32) && !defined(_EMX)
+    if (Help[I]==MCHelpSwAC)
+      continue;
+#endif
 #ifndef SAVE_LINKS
     if (Help[I]==MCHelpSwOL)
       continue;
 #endif
-#if !defined(_WIN_32)
+#ifndef _WIN_32
     if (Help[I]==MCHelpSwRI)
       continue;
 #endif
-#if !defined(_BEOS)
+#ifndef PACK_SMP
+    if (Help[I]==MCHelpSwMT)
+      continue;
+#endif
+#ifndef _BEOS
     if (Help[I]==MCHelpSwEE)
     {
 #if defined(_EMX) && !defined(_DJGPP)
@@ -885,12 +1005,12 @@ void CommandData::OutHelp()
     mprintf(St(Help[I]));
   }
   mprintf("\n");
-  ErrHandler.Exit(0);
+  ErrHandler.Exit(USER_ERROR);
 #endif
 }
 
 
-bool CommandData::ExclCheckArgs(StringList *Args,char *CheckName,bool CheckFullPath)
+bool CommandData::ExclCheckArgs(StringList *Args,char *CheckName,bool CheckFullPath,int MatchMode)
 {
   char *Name=ConvertPath(CheckName,NULL);
   char FullName[NM],*CurName;
@@ -902,12 +1022,12 @@ bool CommandData::ExclCheckArgs(StringList *Args,char *CheckName,bool CheckFullP
     {
       if (*FullName==0)
         ConvertNameToFull(CheckName,FullName);
-      if (CmpName(CurName,FullName,MATCH_WILDSUBPATH))
+      if (CmpName(CurName,FullName,MatchMode))
         return(true);
     }
     else
 #endif
-      if (CmpName(ConvertPath(CurName,NULL),Name,MATCH_WILDSUBPATH))
+      if (CmpName(ConvertPath(CurName,NULL),Name,MatchMode))
         return(true);
   return(false);
 }
@@ -915,11 +1035,11 @@ bool CommandData::ExclCheckArgs(StringList *Args,char *CheckName,bool CheckFullP
 
 bool CommandData::ExclCheck(char *CheckName,bool CheckFullPath)
 {
-  if (ExclCheckArgs(ExclArgs,CheckName,CheckFullPath))
+  if (ExclCheckArgs(ExclArgs,CheckName,CheckFullPath,MATCH_WILDSUBPATH))
     return(true);
   if (InclArgs->ItemsCount()==0)
     return(false);
-  if (ExclCheckArgs(InclArgs,CheckName,CheckFullPath))
+  if (ExclCheckArgs(InclArgs,CheckName,false,MATCH_WILDSUBPATH))
     return(false);
   return(true);
 }
@@ -957,6 +1077,18 @@ bool CommandData::TimeCheck(RarTime &ft)
 #endif
 
 
+#ifndef SFX_MODULE
+bool CommandData::SizeCheck(Int64 Size)
+{
+  if (FileSizeLess!=INT64ERR && Size>=FileSizeLess)
+    return(true);
+  if (FileSizeMore!=INT64ERR && Size<=FileSizeMore)
+    return(true);
+  return(false);
+}
+#endif
+
+
 int CommandData::IsProcessFile(FileHeader &NewLhd,bool *ExactMatch,int MatchType)
 {
   if (strlen(NewLhd.FileName)>=NM || strlenw(NewLhd.FileNameW)>=NM)
@@ -966,6 +1098,10 @@ int CommandData::IsProcessFile(FileHeader &NewLhd,bool *ExactMatch,int MatchType
 #ifndef SFX_MODULE
   if (TimeCheck(NewLhd.mtime))
     return(0);
+  if ((NewLhd.FileAttr & ExclFileAttr)!=0 || InclAttrSet && (NewLhd.FileAttr & InclFileAttr)==0)
+    return(0);
+  if ((NewLhd.Flags & LHD_WINDOWMASK)!=LHD_DIRECTORY && SizeCheck(NewLhd.FullUnpSize))
+    return(0);
 #endif
   char *ArgName;
   wchar *ArgNameW;
@@ -977,14 +1113,17 @@ int CommandData::IsProcessFile(FileHeader &NewLhd,bool *ExactMatch,int MatchType
     if (Unicode)
     {
       wchar NameW[NM],ArgW[NM],*NamePtr=NewLhd.FileNameW;
+      bool CorrectUnicode=true;
       if (ArgNameW==NULL)
       {
-        CharToWide(ArgName,ArgW);
+        if (!CharToWide(ArgName,ArgW) || *ArgW==0)
+          CorrectUnicode=false;
         ArgNameW=ArgW;
       }
       if ((NewLhd.Flags & LHD_UNICODE)==0)
       {
-        CharToWide(NewLhd.FileName,NameW);
+        if (!CharToWide(NewLhd.FileName,NameW) || *NameW==0)
+          CorrectUnicode=false;
         NamePtr=NameW;
       }
       if (CmpName(ArgNameW,NamePtr,MatchType))
@@ -993,7 +1132,8 @@ int CommandData::IsProcessFile(FileHeader &NewLhd,bool *ExactMatch,int MatchType
           *ExactMatch=stricompcw(ArgNameW,NamePtr)==0;
         return(StringCount);
       }
-      continue;
+      if (CorrectUnicode)
+        continue;
     }
 #endif
     if (CmpName(ArgName,NewLhd.FileName,MatchType))
@@ -1007,11 +1147,13 @@ int CommandData::IsProcessFile(FileHeader &NewLhd,bool *ExactMatch,int MatchType
 }
 
 
-#ifndef _WIN_CE
+#ifndef GUI
 void CommandData::ProcessCommand()
 {
 #ifndef SFX_MODULE
-  if (Command[1] && strchr("FUADPXETK",*Command)!=NULL || *ArcName==0)
+
+  const char *SingleCharCommands="FUADPXETK";
+  if (Command[1] && strchr(SingleCharCommands,*Command)!=NULL || *ArcName==0)
     OutHelp();
 
 #ifdef _UNIX
@@ -1047,7 +1189,7 @@ void CommandData::ProcessCommand()
         Extract.DoExtract(this);
       }
       break;
-#if !defined(GUI) && !defined(SILENT)
+#ifndef SILENT
     case 'V':
     case 'L':
       ListArchive(this);
@@ -1056,10 +1198,8 @@ void CommandData::ProcessCommand()
       OutHelp();
 #endif
   }
-#ifndef GUI
   if (!BareOutput)
     mprintf("\n");
-#endif
 }
 #endif
 
@@ -1097,7 +1237,7 @@ uint CommandData::GetExclAttr(char *Str)
   {
     uint Attr;
     for (Attr=0;*Str;Str++)
-      switch(toupper(*Str))
+      switch(etoupper(*Str))
       {
 #ifdef _UNIX
         case 'D':
