@@ -62,7 +62,7 @@ bool FileCreate(RAROptions *Cmd,File *NewFile,char *Name,wchar *NameW,
 #ifdef  _WIN_32
         File SrcFile;
         SrcFile.SetHandleType(FILE_HANDLESTD);
-        int Size=SrcFile.Read(NewName,NM);
+        int Size=SrcFile.Read(NewName,sizeof(NewName)-1);
         NewName[Size]=0;
         OemToChar(NewName,NewName);
 #else
@@ -80,6 +80,17 @@ bool FileCreate(RAROptions *Cmd,File *NewFile,char *Name,wchar *NameW,
       if (Choice==6)
         ErrHandler.Exit(USER_BREAK);
     }
+    if (Mode==OVERWRITE_AUTORENAME)
+    {
+      if (GetAutoRenamedName(Name))
+      {
+        if (NameW!=NULL)
+          *NameW=0;
+      }
+      else
+        Mode=OVERWRITE_ASK;
+      continue;
+    }
   }
   if (NewFile!=NULL && NewFile->Create(Name,NameW))
     return(true);
@@ -89,6 +100,30 @@ bool FileCreate(RAROptions *Cmd,File *NewFile,char *Name,wchar *NameW,
 }
 
 
+bool GetAutoRenamedName(char *Name)
+{
+  char NewName[NM];
+
+  if (strlen(Name)>sizeof(NewName)-10)
+    return(false);
+  char *Ext=GetExt(Name);
+  if (Ext==NULL)
+    Ext=Name+strlen(Name);
+  for (int FileVer=1;;FileVer++)
+  {
+    sprintf(NewName,"%.*s(%d)%s",Ext-Name,Name,FileVer,Ext);
+    if (!FileExist(NewName))
+    {
+      strcpy(Name,NewName);
+      break;
+    }
+    if (FileVer>=1000000)
+      return(false);
+  }
+  return(true);
+}
+
+
 #if defined(_WIN_32) && !defined(_WIN_CE)
 bool UpdateExistingShortName(char *Name,wchar *NameW)
 {
@@ -104,7 +139,7 @@ bool UpdateExistingShortName(char *Name,wchar *NameW)
   char NewName[NM];
   for (int I=0;I<10000;I+=123)
   {
-    strncpy(NewName,Name,sizeof(NewName));
+    strncpyz(NewName,Name,ASIZE(NewName));
     sprintf(PointToName(NewName),"rtmp%d",I);
     if (!FileExist(NewName))
       break;
@@ -112,7 +147,7 @@ bool UpdateExistingShortName(char *Name,wchar *NameW)
   if (FileExist(NewName))
     return(false);
   char FullName[NM];
-  strncpy(FullName,Name,sizeof(FullName));
+  strncpyz(FullName,Name,ASIZE(FullName));
   strcpy(PointToName(FullName),PointToName(fd.Name));
   if (!MoveFile(FullName,NewName))
     return(false);
