@@ -44,12 +44,13 @@ char* ConvertPath(const char *SrcPath,char *DestPath)
     const char *s=DestPtr;
     if (s[0] && IsDriveDiv(s[1]))
       s+=2;
-    if (s[0]=='\\' && s[1]=='\\')
-    {
-      const char *Slash=strchr(s+2,'\\');
-      if (Slash!=NULL && (Slash=strchr(Slash+1,'\\'))!=NULL)
-        s=Slash+1;
-    }
+    else
+      if (s[0]=='\\' && s[1]=='\\')
+      {
+        const char *Slash=strchr(s+2,'\\');
+        if (Slash!=NULL && (Slash=strchr(Slash+1,'\\'))!=NULL)
+          s=Slash+1;
+      }
     for (const char *t=s;*t!=0;t++)
       if (IsPathDiv(*t))
         s=t+1;
@@ -333,6 +334,7 @@ bool EnumConfigPaths(char *Path,int Number)
     strcpy(RARFileName,Path);
   if (Number!=0)
     return(false);
+#ifndef _DJGPP
   if (_osmode==OS2_MODE)
   {
     PTIB ptib;
@@ -341,6 +343,7 @@ bool EnumConfigPaths(char *Path,int Number)
     DosQueryModuleName(ppib->pib_hmte,NM,Path);
   }
   else
+#endif
     strcpy(Path,RARFileName);
   RemoveNameFromPath(Path);
   return(true);
@@ -475,8 +478,12 @@ bool IsNameUsable(const char *Name)
   if (Name[0] && Name[1] && strchr(Name+2,':')!=NULL)
     return(false);
   for (const char *s=Name;*s!=0;s=charnext(s))
+  {
     if (*s<32)
       return(false);
+    if (*s==' ' && IsPathDiv(s[1]))
+      return(false);
+  }
 #endif
   return(*Name!=0 && strpbrk(Name,"?*<>|\"")==NULL);
 }
@@ -495,6 +502,8 @@ void MakeNameUsable(char *Name,bool Extended)
 #ifndef _UNIX
     if (s-Name>1 && *s==':')
       *s='_';
+    if (*s==' ' && IsPathDiv(s[1]))
+      *s='_';
 #endif
   }
 }
