@@ -18,10 +18,10 @@ int uni_init(int codepage);
 int uni_done();
 #endif
 
-bool WideToChar(const wchar *Src,char *Dest,int DestSize=0x10000000);
-bool CharToWide(const char *Src,wchar *Dest,int DestSize=0x10000000);
-byte* WideToRaw(const wchar *Src,byte *Dest,int DestSize=0x10000000);
-wchar* RawToWide(const byte *Src,wchar *Dest,int DestSize=0x10000000);
+bool WideToChar(const wchar *Src,char *Dest,int DestSize=0x1000000);
+bool CharToWide(const char *Src,wchar *Dest,int DestSize=0x1000000);
+byte* WideToRaw(const wchar *Src,byte *Dest,int DestSize=0x1000000);
+wchar* RawToWide(const byte *Src,wchar *Dest,int DestSize=0x1000000);
 void WideToUtf(const wchar *Src,char *Dest,int DestSize);
 void UtfToWide(const char *Src,wchar *Dest,int DestSize);
 bool UnicodeEnabled();
@@ -63,7 +63,7 @@ class SupportDBCS
 extern SupportDBCS gdbcs;
 
 inline char* charnext(const char *s) {return (char *)(gdbcs.DBCSMode ? gdbcs.charnext(s):s+1);}
-inline uint strlend(const char *s) {return (gdbcs.DBCSMode ? gdbcs.strlend(s):strlen(s));}
+inline uint strlend(const char *s) {return (uint)(gdbcs.DBCSMode ? gdbcs.strlend(s):strlen(s));}
 inline char* strchrd(const char *s, int c) {return (char *)(gdbcs.DBCSMode ? gdbcs.strchrd(s,c):strchr(s,c));}
 inline char* strrchrd(const char *s, int c) {return (char *)(gdbcs.DBCSMode ? gdbcs.strrchrd(s,c):strrchr(s,c));}
 inline void copychrd(char *dest,const char *src) {if (gdbcs.DBCSMode) gdbcs.copychrd(dest,src); else *dest=*src;}
